@@ -1,44 +1,32 @@
 import collections.abc
 import os
 import pickle
-import re
-import string
 import sys
 from copy import copy, deepcopy
-from typing import Callable, Dict, ForwardRef, List, NewType, Tuple, TypeVar, Union
+from typing import Callable, Dict, Generic, List, NewType, Tuple, TypeVar, Union
 
 import pytest
+from dirty_equals import IsList
+from pydantic_core import PydanticCustomError, PydanticUndefined, core_schema
 from typing_extensions import Annotated, Literal
 
-from pydantic import BaseModel, ConstrainedList, conlist
-from pydantic.color import Color
-from pydantic.dataclasses import dataclass
-from pydantic.fields import Undefined
-from pydantic.typing import (
-    all_literal_values,
-    display_as_type,
-    get_args,
-    get_origin,
-    is_new_type,
-    new_type_supertype,
-    resolve_annotations,
-)
-from pydantic.utils import (
+from pydantic import BaseModel
+from pydantic._internal import _repr
+from pydantic._internal._core_utils import _WalkCoreSchema
+from pydantic._internal._typing_extra import all_literal_values, get_origin, is_new_type
+from pydantic._internal._utils import (
     BUILTIN_COLLECTIONS,
     ClassAttribute,
     ValueItems,
     all_identical,
     deep_update,
-    get_model,
-    import_string,
     lenient_issubclass,
-    path_type,
     smart_deepcopy,
-    to_lower_camel,
-    truncate,
     unique_list,
 )
-from pydantic.version import version_info
+from pydantic._internal._validators import import_string
+from pydantic.alias_generators import to_camel, to_pascal, to_snake
+from pydantic.color import Color
 
 try:
     import devtools
@@ -51,27 +39,73 @@ def test_import_module():
 
 
 def test_import_module_invalid():
-    with pytest.raises(ImportError) as exc_info:
+    with pytest.raises(PydanticCustomError, match="Invalid python path: No module named 'xx'"):
         import_string('xx')
-    assert exc_info.value.args[0] == '"xx" doesn\'t look like a module path'
 
 
 def test_import_no_attr():
-    with pytest.raises(ImportError) as exc_info:
-        import_string('os.foobar')
-    assert exc_info.value.args[0] == 'Module "os" does not define a "foobar" attribute'
+    with pytest.raises(PydanticCustomError, match="cannot import name 'foobar' from 'os'"):
+        import_string('os:foobar')
+
+
+def foobar(a, b, c=4):
+    pass
+
+
+T = TypeVar('T')
+
+
+class LoggedVar(Generic[T]):
+    def get(self) -> T:
+        ...
 
 
 @pytest.mark.parametrize(
-    'value,expected', ((str, 'str'), ('string', 'str'), (Union[str, int], 'Union[str, int]'), (list, 'list'))
+    'value,expected',
+    [
+        (str, 'str'),
+        ('foobar', 'str'),
+        ('SomeForwardRefString', 'str'),  # included to document current behavior; could be changed
+        (List['SomeForwardRef'], "List[ForwardRef('SomeForwardRef')]"),  # noqa: F821
+        (Union[str, int], 'Union[str, int]'),
+        (list, 'list'),
+        (List, 'List'),
+        ([1, 2, 3], 'list'),
+        (List[Dict[str, int]], 'List[Dict[str, int]]'),
+        (Tuple[str, int, float], 'Tuple[str, int, float]'),
+        (Tuple[str, ...], 'Tuple[str, ...]'),
+        (Union[int, List[str], Tuple[str, int]], 'Union[int, List[str], Tuple[str, int]]'),
+        (foobar, 'foobar'),
+        (LoggedVar, 'LoggedVar'),
+        (LoggedVar(), 'LoggedVar'),
+    ],
 )
 def test_display_as_type(value, expected):
-    assert display_as_type(value) == expected
+    assert _repr.display_as_type(value) == expected
 
 
-@pytest.mark.skipif(sys.version_info < (3, 9), reason='generic aliases are not available in python < 3.9')
-def test_display_as_type_generic_alias():
-    assert display_as_type(list[[Union[str, int]]]) == 'list[[Union[str, int]]]'
+@pytest.mark.skipif(sys.version_info < (3, 10), reason='requires python 3.10 or higher')
+@pytest.mark.parametrize(
+    'value_gen,expected',
+    [
+        (lambda: str, 'str'),
+        (lambda: 'SomeForwardRefString', 'str'),  # included to document current behavior; could be changed
+        (lambda: List['SomeForwardRef'], "List[ForwardRef('SomeForwardRef')]"),  # noqa: F821
+        (lambda: str | int, 'Union[str, int]'),
+        (lambda: list, 'list'),
+        (lambda: List, 'List'),
+        (lambda: list[int], 'list[int]'),
+        (lambda: List[int], 'List[int]'),
+        (lambda: list[dict[str, int]], 'list[dict[str, int]]'),
+        (lambda: list[Union[str, int]], 'list[Union[str, int]]'),
+        (lambda: list[str | int], 'list[Union[str, int]]'),
+        (lambda: LoggedVar[int], 'LoggedVar[int]'),
+        (lambda: LoggedVar[Dict[int, str]], 'LoggedVar[Dict[int, str]]'),
+    ],
+)
+def test_display_as_type_310(value_gen, expected):
+    value = value_gen()
+    assert _repr.display_as_type(value) == expected
 
 
 def test_lenient_issubclass():
@@ -93,19 +127,6 @@ def test_lenient_issubclass_is_lenient():
     assert lenient_issubclass('a', 'a') is False
 
 
-@pytest.mark.parametrize(
-    'input_value,output',
-    [
-        (object, "<class 'object'>"),
-        (string.ascii_lowercase, "'abcdefghijklmnopq…'"),
-        (list(range(20)), '[0, 1, 2, 3, 4, 5, …'),
-    ],
-)
-def test_truncate(input_value, output):
-    with pytest.warns(DeprecationWarning, match='`truncate` is no-longer used by pydantic and is deprecated'):
-        assert truncate(input_value, max_len=20) == output
-
-
 @pytest.mark.parametrize(
     'input_value,output',
     [
@@ -156,6 +177,25 @@ def test_value_items():
     assert sub_vi.is_included(2)
     assert [v_ for i, v_ in enumerate(sub_v) if sub_vi.is_included(i)] == ['a', 'c']
 
+    vi = ValueItems([], {'__all__': {}})
+    assert vi._items == {}
+
+    with pytest.raises(TypeError, match='Unexpected type of exclude value for index "a" <class \'NoneType\'>'):
+        ValueItems(['a', 'b'], {'a': None})
+
+    m = (
+        'Excluding fields from a sequence of sub-models or dicts must be performed index-wise: '
+        'expected integer keys or keyword "__all__"'
+    )
+    with pytest.raises(TypeError, match=m):
+        ValueItems(['a', 'b'], {'a': {}})
+
+    vi = ValueItems([1, 2, 3, 4], {'__all__': True})
+    assert repr(vi) == 'ValueItems({0: Ellipsis, 1: Ellipsis, 2: Ellipsis, 3: Ellipsis})'
+
+    vi = ValueItems([1, 2], {'__all__': {1, 2}})
+    assert repr(vi) == 'ValueItems({0: {1: Ellipsis, 2: Ellipsis}, 1: {1: Ellipsis, 2: Ellipsis}})'
+
 
 @pytest.mark.parametrize(
     'base,override,intersect,expected',
@@ -221,17 +261,10 @@ def test_is_new_type():
     assert not is_new_type(str)
 
 
-def test_new_type_supertype():
-    new_type = NewType('new_type', str)
-    new_new_type = NewType('new_new_type', new_type)
-    assert new_type_supertype(new_type) == str
-    assert new_type_supertype(new_new_type) == str
-
-
 def test_pretty():
     class MyTestModel(BaseModel):
-        a = 1
-        b = [1, 2, 3]
+        a: int = 1
+        b: List[int] = [1, 2, 3]
 
     m = MyTestModel()
     assert m.__repr_name__() == 'MyTestModel'
@@ -253,6 +286,7 @@ def test_pretty():
     ]
 
 
+@pytest.mark.filterwarnings('ignore::DeprecationWarning')
 def test_pretty_color():
     c = Color('red')
     assert str(c) == 'red'
@@ -275,33 +309,12 @@ def test_pretty_color():
 @pytest.mark.skipif(not devtools, reason='devtools not installed')
 def test_devtools_output():
     class MyTestModel(BaseModel):
-        a = 1
-        b = [1, 2, 3]
+        a: int = 1
+        b: List[int] = [1, 2, 3]
 
     assert devtools.pformat(MyTestModel()) == 'MyTestModel(\n    a=1,\n    b=[1, 2, 3],\n)'
 
 
-@pytest.mark.skipif(not devtools, reason='devtools not installed')
-def test_devtools_output_validation_error():
-    class Model(BaseModel):
-        a: int
-
-    with pytest.raises(ValueError) as exc_info:
-        Model()
-    assert devtools.pformat(exc_info.value) == (
-        'ValidationError(\n'
-        "    model='Model',\n"
-        '    errors=[\n'
-        '        {\n'
-        "            'loc': ('a',),\n"
-        "            'msg': 'field required',\n"
-        "            'type': 'value_error.missing',\n"
-        '        },\n'
-        '    ],\n'
-        ')'
-    )
-
-
 @pytest.mark.parametrize(
     'mapping, updating_mapping, expected_mapping, msg',
     [
@@ -343,37 +356,12 @@ def test_deep_update_is_not_mutating():
 
 
 def test_undefined_repr():
-    assert repr(Undefined) == 'PydanticUndefined'
+    assert repr(PydanticUndefined) == 'PydanticUndefined'
 
 
 def test_undefined_copy():
-    assert copy(Undefined) is Undefined
-    assert deepcopy(Undefined) is Undefined
-
-
-def test_get_model():
-    class A(BaseModel):
-        a: str
-
-    assert get_model(A) == A
-
-    @dataclass
-    class B:
-        a: str
-
-    assert get_model(B) == B.__pydantic_model__
-
-    class C:
-        pass
-
-    with pytest.raises(TypeError):
-        get_model(C)
-
-
-def test_version_info():
-    s = version_info()
-    assert re.match(' *pydantic version: ', s)
-    assert s.count('\n') == 5
+    assert copy(PydanticUndefined) is PydanticUndefined
+    assert deepcopy(PydanticUndefined) is PydanticUndefined
 
 
 def test_class_attribute():
@@ -392,40 +380,14 @@ def test_class_attribute():
 
 def test_all_literal_values():
     L1 = Literal['1']
-    assert all_literal_values(L1) == ('1',)
+    assert all_literal_values(L1) == ['1']
 
     L2 = Literal['2']
     L12 = Literal[L1, L2]
-    assert sorted(all_literal_values(L12)) == sorted(('1', '2'))
+    assert all_literal_values(L12) == IsList('1', '2', check_order=False)
 
     L312 = Literal['3', Literal[L1, L2]]
-    assert sorted(all_literal_values(L312)) == sorted(('1', '2', '3'))
-
-
-def test_path_type(tmp_path):
-    assert path_type(tmp_path) == 'directory'
-    file = tmp_path / 'foobar.txt'
-    file.write_text('hello')
-    assert path_type(file) == 'file'
-
-
-def test_path_type_unknown(tmp_path):
-    p = type(
-        'FakePath',
-        (),
-        {
-            'exists': lambda: True,
-            'is_dir': lambda: False,
-            'is_file': lambda: False,
-            'is_mount': lambda: False,
-            'is_symlink': lambda: False,
-            'is_block_device': lambda: False,
-            'is_char_device': lambda: False,
-            'is_fifo': lambda: False,
-            'is_socket': lambda: False,
-        },
-    )
-    assert path_type(p) == 'unknown'
+    assert all_literal_values(L312) == IsList('3', '1', '2', check_order=False)
 
 
 @pytest.mark.parametrize(
@@ -435,13 +397,13 @@ def test_path_type_unknown(tmp_path):
 def test_smart_deepcopy_immutable_non_sequence(obj, mocker):
     # make sure deepcopy is not used
     # (other option will be to use obj.copy(), but this will produce error as none of given objects have this method)
-    mocker.patch('pydantic.utils.deepcopy', side_effect=RuntimeError)
+    mocker.patch('pydantic._internal._utils.deepcopy', side_effect=RuntimeError)
     assert smart_deepcopy(obj) is deepcopy(obj) is obj
 
 
 @pytest.mark.parametrize('empty_collection', (collection() for collection in BUILTIN_COLLECTIONS))
 def test_smart_deepcopy_empty_collection(empty_collection, mocker):
-    mocker.patch('pydantic.utils.deepcopy', side_effect=RuntimeError)  # make sure deepcopy is not used
+    mocker.patch('pydantic._internal._utils.deepcopy', side_effect=RuntimeError)  # make sure deepcopy is not used
     if not isinstance(empty_collection, (tuple, frozenset)):  # empty tuple or frozenset are always the same object
         assert smart_deepcopy(empty_collection) is not empty_collection
 
@@ -451,7 +413,7 @@ def test_smart_deepcopy_empty_collection(empty_collection, mocker):
 )
 def test_smart_deepcopy_collection(collection, mocker):
     expected_value = object()
-    mocker.patch('pydantic.utils.deepcopy', return_value=expected_value)
+    mocker.patch('pydantic._internal._utils.deepcopy', return_value=expected_value)
     assert smart_deepcopy(collection) is expected_value
 
 
@@ -486,33 +448,6 @@ def test_get_origin(input_value, output_value):
     assert get_origin(input_value) is output_value
 
 
-@pytest.mark.skipif(sys.version_info < (3, 8), reason='get_args is only consistent for python >= 3.8')
-@pytest.mark.parametrize(
-    'input_value,output_value',
-    [
-        (conlist(str), (str,)),
-        (ConstrainedList, ()),
-        (List[str], (str,)),
-        (Dict[str, int], (str, int)),
-        (int, ()),
-        (Union[int, Union[T, int], str][int], (int, str)),
-        (Union[int, Tuple[T, int]][str], (int, Tuple[str, int])),
-        (Callable[[], T][int], ([], int)),
-        (Annotated[int, 10] if Annotated else None, (int, 10)),
-    ],
-)
-def test_get_args(input_value, output_value):
-    if input_value is None:
-        pytest.skip('Skipping undefined hint for this python version')
-    assert get_args(input_value) == output_value
-
-
-def test_resolve_annotations_no_module():
-    # TODO: is there a better test for this, can this case really happen?
-    fr = ForwardRef('Foo')
-    assert resolve_annotations({'Foo': ForwardRef('Foo')}, None) == {'Foo': fr}
-
-
 def test_all_identical():
     a, b = object(), object()
     c = [b]
@@ -529,17 +464,214 @@ def test_all_identical():
 
 
 def test_undefined_pickle():
-    undefined2 = pickle.loads(pickle.dumps(Undefined))
-    assert undefined2 is Undefined
+    undefined2 = pickle.loads(pickle.dumps(PydanticUndefined))
+    assert undefined2 is PydanticUndefined
 
 
 def test_on_lower_camel_zero_length():
-    assert to_lower_camel('') == ''
+    assert to_camel('') == ''
 
 
 def test_on_lower_camel_one_length():
-    assert to_lower_camel('a') == 'a'
+    assert to_camel('a') == 'a'
 
 
 def test_on_lower_camel_many_length():
-    assert to_lower_camel('i_like_turtles') == 'iLikeTurtles'
+    assert to_camel('i_like_turtles') == 'iLikeTurtles'
+
+
+@pytest.mark.parametrize(
+    'value,result',
+    [
+        ('snake_to_camel', 'snakeToCamel'),
+        ('snake_2_camel', 'snake2Camel'),
+        ('snake2camel', 'snake2Camel'),
+        ('_snake_to_camel', '_snakeToCamel'),
+        ('snake_to_camel_', 'snakeToCamel_'),
+        ('__snake_to_camel__', '__snakeToCamel__'),
+        ('snake_2', 'snake2'),
+        ('_snake_2', '_snake2'),
+        ('snake_2_', 'snake2_'),
+    ],
+)
+def test_snake2camel_start_lower(value: str, result: str) -> None:
+    assert to_camel(value) == result
+
+
+@pytest.mark.parametrize(
+    'value,result',
+    [
+        ('snake_to_camel', 'SnakeToCamel'),
+        ('snake_2_camel', 'Snake2Camel'),
+        ('snake2camel', 'Snake2Camel'),
+        ('_snake_to_camel', '_SnakeToCamel'),
+        ('snake_to_camel_', 'SnakeToCamel_'),
+        ('__snake_to_camel__', '__SnakeToCamel__'),
+        ('snake_2', 'Snake2'),
+        ('_snake_2', '_Snake2'),
+        ('snake_2_', 'Snake2_'),
+    ],
+)
+def test_snake2camel(value: str, result: str) -> None:
+    assert to_pascal(value) == result
+
+
+@pytest.mark.parametrize(
+    'value,result',
+    [
+        ('camel_to_snake', 'camel_to_snake'),
+        ('camelToSnake', 'camel_to_snake'),
+        ('camel2Snake', 'camel_2_snake'),
+        ('_camelToSnake', '_camel_to_snake'),
+        ('camelToSnake_', 'camel_to_snake_'),
+        ('__camelToSnake__', '__camel_to_snake__'),
+        ('CamelToSnake', 'camel_to_snake'),
+        ('Camel2Snake', 'camel_2_snake'),
+        ('_CamelToSnake', '_camel_to_snake'),
+        ('CamelToSnake_', 'camel_to_snake_'),
+        ('__CamelToSnake__', '__camel_to_snake__'),
+        ('Camel2', 'camel_2'),
+        ('Camel2_', 'camel_2_'),
+        ('_Camel2', '_camel_2'),
+        ('camel2', 'camel_2'),
+        ('camel2_', 'camel_2_'),
+        ('_camel2', '_camel_2'),
+    ],
+)
+def test_camel2snake(value: str, result: str) -> None:
+    assert to_snake(value) == result
+
+
+@pytest.mark.parametrize(
+    'params,expected_extra_schema',
+    (
+        pytest.param({}, {}, id='Positional tuple without extra_schema'),
+        pytest.param(
+            {'extra_schema': core_schema.float_schema()},
+            {'extra_schema': {'type': 'str'}},
+            id='Positional tuple with extra_schema',
+        ),
+    ),
+)
+def test_handle_tuple_positional_schema(params, expected_extra_schema):
+    schema = core_schema.tuple_positional_schema([core_schema.str_schema()], **params)
+
+    def walk(s, recurse):
+        # change extra_schema['type'] to 'str'
+        if s['type'] == 'float':
+            s['type'] = 'str'
+        return s
+
+    schema = _WalkCoreSchema().handle_tuple_positional_schema(schema, walk)
+    assert schema == {
+        **expected_extra_schema,
+        'items_schema': [{'type': 'str'}],
+        'type': 'tuple-positional',
+    }
+
+
+@pytest.mark.parametrize(
+    'params,expected_extra_schema',
+    (
+        pytest.param({}, {}, id='Model fields without extra_validator'),
+        pytest.param(
+            {'extra_validator': core_schema.float_schema()},
+            {'extra_validator': {'type': 'str'}},
+            id='Model fields with extra_validator',
+        ),
+    ),
+)
+def test_handle_model_fields_schema(params, expected_extra_schema):
+    schema = core_schema.model_fields_schema(
+        {
+            'foo': core_schema.model_field(core_schema.int_schema()),
+        },
+        **params,
+    )
+
+    def walk(s, recurse):
+        # change extra_schema['type'] to 'str'
+        if s['type'] == 'float':
+            s['type'] = 'str'
+        return s
+
+    schema = _WalkCoreSchema().handle_model_fields_schema(schema, walk)
+    assert schema == {
+        **expected_extra_schema,
+        'type': 'model-fields',
+        'fields': {'foo': {'type': 'model-field', 'schema': {'type': 'int'}}},
+    }
+
+
+@pytest.mark.parametrize(
+    'params,expected_extra_schema',
+    (
+        pytest.param({}, {}, id='Typeddict without extra_validator'),
+        pytest.param(
+            {'extra_validator': core_schema.float_schema()},
+            {'extra_validator': {'type': 'str'}},
+            id='Typeddict with extra_validator',
+        ),
+    ),
+)
+def test_handle_typed_dict_schema(params, expected_extra_schema):
+    schema = core_schema.typed_dict_schema(
+        {
+            'foo': core_schema.model_field(core_schema.int_schema()),
+        },
+        **params,
+    )
+
+    def walk(s, recurse):
+        # change extra_validator['type'] to 'str'
+        if s['type'] == 'float':
+            s['type'] = 'str'
+        return s
+
+    schema = _WalkCoreSchema().handle_typed_dict_schema(schema, walk)
+    assert schema == {
+        **expected_extra_schema,
+        'type': 'typed-dict',
+        'fields': {'foo': {'type': 'model-field', 'schema': {'type': 'int'}}},
+    }
+
+
+def test_handle_function_schema():
+    schema = core_schema.field_before_validator_function(function=lambda v, _info: v, schema=core_schema.float_schema())
+
+    def walk(s, recurse):
+        # change type to str
+        if s['type'] == 'float':
+            s['type'] = 'str'
+        return s
+
+    schema = _WalkCoreSchema().handle_function_schema(schema, walk)
+    assert schema['type'] == 'function-before'
+    assert schema['schema'] == {'type': 'str'}
+
+    def walk1(s, recurse):
+        # this is here to make sure this function is not called
+        assert False
+
+    schema = _WalkCoreSchema().handle_function_schema(core_schema.int_schema(), walk1)
+    assert schema['type'] == 'int'
+
+
+def test_handle_call_schema():
+    param_a = core_schema.arguments_parameter(name='a', schema=core_schema.str_schema(), mode='positional_only')
+    args_schema = core_schema.arguments_schema([param_a])
+
+    schema = core_schema.call_schema(
+        arguments=args_schema,
+        function=lambda a: int(a),
+        return_schema=core_schema.str_schema(),
+    )
+
+    def walk(s, recurse):
+        # change return schema
+        if 'return_schema' in schema:
+            schema['return_schema']['type'] = 'int'
+        return s
+
+    schema = _WalkCoreSchema().handle_call_schema(schema, walk)
+    assert schema['return_schema'] == {'type': 'int'}
